﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.Diagnostics;
using System.Threading;
using System.Text.RegularExpressions;

namespace NexusUpdater
{
    public partial class ConfirmDlg : Form
    {
        private int m_nPhase = 0;
        private string m_sMessage = string.Empty;
        private AutoResetEvent m_event = new AutoResetEvent(false);
        private Process m_currentProc = null;

        public string BasePath
        {
            get;
            set;
        }

        private string PathFastBoot
        {
            get
            {
                return Path.Combine(BasePath, "fastboot.exe");
            }
        }

        private string PathFlashAll
        {
            get
            {
                return Path.Combine(BasePath, "flash-all.bat");
            }
        }

        private bool IsHasFlashAll
        {
            get
            {
                return File.Exists(PathFlashAll);
            }
        }

        public ConfirmDlg()
        {
            InitializeComponent();
        }

        private void ConfirmDlg_Load(object sender, EventArgs e)
        {
            DisplayMessage();
        }       

        private void btnNext_Click(object sender, EventArgs e)
        {
            m_nPhase++;
            DisplayMessage();
            switch (m_nPhase)
            {
                case 0:
                   
                    break;
                case 1:
                    Exec_Unlock();
                    break;
                case 2:
                    break;
                case 3:
                    if (IsHasFlashAll)
                        Exec_UpdateDirect(PathFastBoot);
                    else
                        Exec_UpdateInDirect();
                    break;
                case 6:
                    Exec_Lock();
                    break;
            }            
        }

        private void SetMsgText()
        {
            if (this.InvokeRequired)
            {
                this.Invoke(new MethodInvoker(SetMsgText));
            }
            else
            {
                txtMessage.Text = m_sMessage;
                txtMessage.Select(0, 0);
            }
        }

        private void DisplayMessage()
        {
            StringBuilder sbMessage = new StringBuilder();
            switch (m_nPhase)
            {
                case 0:
                    sbMessage.AppendLine(" ** 준비 작업 ** ");
                    sbMessage.AppendLine();
                    sbMessage.AppendLine("먼저 휴대폰의 전원을 끕니다.");
                    sbMessage.AppendLine();
                    sbMessage.AppendLine("컴퓨터와 휴대폰을 USB로 연결합니다");
                    sbMessage.AppendLine();
                    sbMessage.AppendLine("그리고 난 뒤, 휴대폰을 다시 켤때, 볼륨 UP + 전원버튼 을 누르고 켭니다.");
                    sbMessage.AppendLine();
                    sbMessage.AppendLine("관리 화면이 떴으면 '다음' 버튼을 누르세요");
                    picBox.Image = Properties.Resources.fastboot_screen;
                    break;
                case 1:
                    sbMessage.AppendLine(" ** Nexus Unlcok ** ");
                    sbMessage.AppendLine();
                    sbMessage.AppendLine("업데이트 하려면 Unlock이라는 것을 시도해야 합니다.");
                    sbMessage.AppendLine();
                    sbMessage.AppendLine("현재 Lock이 걸려있던 휴대폰이면,");
                    sbMessage.AppendLine();
                    sbMessage.AppendLine("휴대폰에서 오른편의 화면처럼 뜨게 될 겁니다");
                    sbMessage.AppendLine();
                    sbMessage.AppendLine("그 때는 휴대폰의 볼륨 조절 버튼을 눌러 'Yes'를 선택하고,");
                    sbMessage.AppendLine();
                    sbMessage.AppendLine("전원 버튼을 눌러주세요.");
                    picBox.Image = Properties.Resources.unlock_confirm;
                    break;
                case 2:
                    sbMessage.AppendLine(" ** Rom Update Ready ** ");
                    sbMessage.AppendLine();
                    sbMessage.AppendLine("오른편 화면과 같이 LOCK STATE 가 UNLOCK으로 되었는지만 확인해주세요.");
                    sbMessage.AppendLine();
                    sbMessage.AppendLine("만일 그렇지 않으면, 이 프로그램을 종료하시고,");
                    sbMessage.AppendLine();
                    sbMessage.AppendLine("처음 부터 다시 수행해주시기 바랍니다.");
                    sbMessage.AppendLine();
                    sbMessage.AppendLine("업데이트 준비되었으면 '다음' 버튼을 누르세요");

                    picBox.Image = Properties.Resources.unlocked;
                    break;
                case 3:
                    sbMessage.AppendLine(" ** Rom Update ** ");
                    sbMessage.AppendLine();
                    sbMessage.AppendLine("롬을 업데이트 합니다.");
                    sbMessage.AppendLine();
                    sbMessage.AppendLine("자동으로 수행하며, 업데이트 중에 휴대폰이 자동으로 리셋됩니다.");
                    sbMessage.AppendLine();
                    picBox.Image = Properties.Resources.update_img;
                    break;
                case 4:
                    sbMessage.AppendLine(" ** Rom Update Complete ** ");
                    sbMessage.AppendLine();
                    sbMessage.AppendLine("롬을 업데이트를 종료합니다.");
                    sbMessage.AppendLine();
                    sbMessage.AppendLine("자동으로 전원이 켜지면 원하는 롬으로 덮어써져 있을 것입니다.");
                    sbMessage.AppendLine();
                    sbMessage.AppendLine("만일 Unlock되어 있는 휴대폰을 Lock 상태로 만들고 싶으면");
                    sbMessage.AppendLine();
                    sbMessage.AppendLine("'다음' 버튼을 누르시고, Unlock상태로 두시려면, 프로그램을 종료하세요");
                    sbMessage.AppendLine();
                    picBox.Image = Properties.Resources.update_img;
                    break;
                case 5:
                    sbMessage.AppendLine(" ** Nexus Lcok ** ");
                    sbMessage.AppendLine();
                    sbMessage.AppendLine("다시 전원을 끈 뒤, 맨 처음 처럼.");
                    sbMessage.AppendLine();
                    sbMessage.AppendLine("휴대폰을 다시 켤때, 볼륨 UP + 전원버튼 을 누르고 켭니다.");
                    sbMessage.AppendLine();
                    sbMessage.AppendLine("관리 화면이 떴으면 '다음' 버튼을 누르세요");
                    sbMessage.AppendLine();
                    picBox.Image = Properties.Resources.fastboot_screen;
                    break;
                case 6:
                    sbMessage.AppendLine(" ** Complete ** ");
                    sbMessage.AppendLine();
                    sbMessage.AppendLine("모든 처리는 끝났습니다.");
                    sbMessage.AppendLine();
                    sbMessage.AppendLine("이제 이전처럼 사용하시고,");
                    sbMessage.AppendLine();
                    sbMessage.AppendLine("원래대로 복원하시기 바랍니다.");
                    sbMessage.AppendLine();
                    picBox.Image = Properties.Resources.complete;
                    break;

            }
            m_sMessage = sbMessage.ToString();
            SetMsgText();
        }

        private void Exec_Unlock()
        {
            try
            {
                btnNext.Enabled = false;
                ProcessStartInfo info = new ProcessStartInfo();
                info.FileName = PathFastBoot;
                info.Arguments = "oem unlock";
                info.WorkingDirectory = this.BasePath;
                info.ErrorDialog = false;
                info.CreateNoWindow = false;
                info.WindowStyle = ProcessWindowStyle.Minimized;
                info.UseShellExecute = true;

                m_currentProc = new Process();
                m_currentProc.StartInfo = info;
                m_currentProc.Start();
                m_currentProc.WaitForExit();

                btnNext_Click(btnNext, null);
                btnNext.Enabled = true;
            }
            catch (Exception ex)
            {
                Logger.Current.Write(ex);
            }
        }

        private void Exec_UpdateDirect(string sPathFastBoot)
        {
            try
            {
                btnNext.Enabled = false;
                ProcessStartInfo info = new ProcessStartInfo();
                info.FileName = sPathFastBoot;
                info.WorkingDirectory = this.BasePath;
                info.ErrorDialog = false;
                info.CreateNoWindow = false;
                info.WindowStyle = ProcessWindowStyle.Minimized;
                info.UseShellExecute = true;

                m_currentProc = new Process();
                m_currentProc.StartInfo = info;
                m_currentProc.Start();
                m_currentProc.WaitForExit();

                btnNext_Click(btnNext, null);
                btnNext.Enabled = true;
            }
            catch (Exception ex)
            {
                Logger.Current.Write(ex);
            }
        }

        private void Exec_UpdateInDirect()
        {
            try
            {
                string sFlashAllFile = Path.Combine(BasePath, "flash-all.sh");
                string sFlashAllBatFile = Path.Combine(BasePath, "flash-all.bat");
                using (StreamReader reader = new StreamReader(File.Open(sFlashAllFile, FileMode.Open)))
                using (StreamWriter writer = new StreamWriter(File.Create(sFlashAllBatFile)))
                {
                    while (reader.EndOfStream == false)
                    {
                        string sReadString = reader.ReadLine();
                        Console.WriteLine(sReadString);
                        if (sReadString.StartsWith("#"))
                            continue;
                        if (sReadString.StartsWith("sleep"))
                            sReadString = "ping -n 5 127.0.0.1 >nul";
                        writer.WriteLine(sReadString);
                    }
                }

                Exec_UpdateDirect(sFlashAllBatFile);
            }
            catch (Exception ex)
            {
                Logger.Current.Write(ex);
            }
        }


        private void Exec_Lock()
        {
            try
            {
                btnNext.Enabled = false;
                ProcessStartInfo info = new ProcessStartInfo();
                info.FileName = PathFastBoot;
                info.Arguments = "oem lock";
                info.WorkingDirectory = this.BasePath;
                info.ErrorDialog = false;
                info.CreateNoWindow = false;
                info.WindowStyle = ProcessWindowStyle.Minimized;
                info.UseShellExecute = true;

                m_currentProc = new Process();
                m_currentProc.StartInfo = info;
                m_currentProc.Start();
                m_currentProc.WaitForExit();

                info = new ProcessStartInfo();
                info.FileName = PathFastBoot;
                info.Arguments = "reboot";
                info.WorkingDirectory = this.BasePath;
                info.ErrorDialog = false;
                info.CreateNoWindow = false;
                info.WindowStyle = ProcessWindowStyle.Minimized;
                info.UseShellExecute = true;

                m_currentProc = new Process();
                m_currentProc.StartInfo = info;
                m_currentProc.Start();
                m_currentProc.WaitForExit();

                btnNext_Click(btnNext, null);
                btnNext.Enabled = true;
            }
            catch (Exception ex)
            {
                Logger.Current.Write(ex);
            }
        }
    }
}
