﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Text;
using System.Text.RegularExpressions;


namespace NexusUpdater
{
    class ImageInfo
    {
        bool bIsChecked = false;

        Regex regexVer = new Regex("<td>(?<ver>[0-9A-Za-z.() ]+)", RegexOptions.Singleline);
        Regex regexLnk = new Regex("<td><a href=\\\"(?<url>[A-Z\\/0-9a-z\\.\\-\\:]+)\\\">Link</a>", RegexOptions.Singleline);
        Regex regexChkSum = new Regex("<td>(?<chksum>[0-9a-z]+)", RegexOptions.Singleline);

        public string ID
        {
            get;
            set;
        }

        public string Name
        {
            get;
            set;
        }

        public string Version
        {
            get;
            set;
        }

        public string Url
        {
            get;
            set;
        }

        public string MD5
        {
            get;
            set;
        }

        public string SHA1
        {
            get;
            set;
        }

        public string Filename
        {
            get;
            set;
        }

        public string FullFilename
        {
            get;
            set;
        }

        public string ExtractedPath
        {
            get;
            set;
        }

        public bool IsDownloaded
        {
            get
            {
                if (File.Exists(this.FullFilename))
                {
                    if (bIsChecked)
                        return true;

                    System.Security.Cryptography.MD5 md5 = System.Security.Cryptography.MD5.Create();

                    using (BufferedStream stream = new BufferedStream(File.OpenRead(this.FullFilename), 1200000))
                    {
                        byte[] retval = md5.ComputeHash(stream);
                        StringBuilder sb = new StringBuilder();
                        for (int i = 0; i < retval.Length; i++)
                        {
                            sb.Append(retval[i].ToString("x2"));
                        }

                        if (sb.ToString().Equals(this.MD5))
                        {
                            bIsChecked = true;
                            return true;
                        }
                    }
                }
                return false;

            }
        }

        public bool IsHasExtracted
        {
            get
            {
                if (string.IsNullOrEmpty(ExtractedPath) == false)
                {
                    System.IO.DirectoryInfo info = new System.IO.DirectoryInfo(ExtractedPath);
                    return (info.Exists && info.GetFiles().Length > 3);
                }
                return false;
            }

        }

        public override string ToString()
        {
            StringBuilder sbName = new StringBuilder();
            if (IsDownloaded)
                sbName.Append("*** ");
            sbName.Append(this.Name);
            sbName.Append("  (");
            sbName.Append(this.Version);
            sbName.Append(")");
            return sbName.ToString();
        }

        public bool Load(string sPath, string sTitle, string sID,  List<string> aryContents)
        {
            bool bResult = false;
            if (aryContents.Count == 4)
            {
                Match matchVer = regexVer.Match(aryContents[0]);
                Match matchLnk = regexLnk.Match(aryContents[1]);
                Match matchMD5 = regexChkSum.Match(aryContents[2]);
                Match matchSHA1 = regexChkSum.Match(aryContents[3]);

                bResult = (matchVer.Success
                    && matchLnk.Success
                    && matchMD5.Success
                    && matchSHA1.Success);
                if(bResult)
                {
                    this.Name = sTitle;
                    this.ID = sID;

                    this.Version = matchVer.Groups["ver"].Value;
                    this.Url = matchLnk.Groups["url"].Value;
                    this.MD5 = matchMD5.Groups["chksum"].Value;
                    this.SHA1 = matchSHA1.Groups["chksum"].Value;
                    
                    Uri uri = new Uri(this.Url);
                    this.Filename = uri.Segments[uri.Segments.Length - 1];
                    string[] extractPath = this.Filename.Split("-".ToCharArray(), StringSplitOptions.RemoveEmptyEntries);
                    this.ExtractedPath = extractPath[0] + "-" + extractPath[1];
                    this.FullFilename = Path.Combine(sPath, this.Filename);
                    if (IsDownloaded == false && File.Exists(this.FullFilename))
                    {
                        File.Delete(this.FullFilename);
                    }
                }
            }



            return bResult;
        }
    }
}
