﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Text.RegularExpressions;
using System.Windows.Forms;
using System.Net;
using System.IO;
using System.Diagnostics;

using ICSharpCode.SharpZipLib.Core;
using ICSharpCode.SharpZipLib.GZip;
using ICSharpCode.SharpZipLib.Tar;
using ICSharpCode.SharpZipLib.Zip;

namespace NexusUpdater
{
    public partial class frmMain : Form
    {
        private Stopwatch m_stopwatch = new Stopwatch();
        private ImageInfo m_selectedImage = null;
        private int m_nPercentage = 0;
        private int m_nPrevPercentage = -1;
        private long m_lnTotal = 0;
        private long m_lnRecv = 0;
        private int m_nP = -1;

        private const string SRC_URL = "https://developers.google.com/android/nexus/images?hl=ko-KR";
        private string m_sPath = string.Empty;
        private string m_sFilename = string.Empty;
        private string m_sPageSrc = string.Empty;
        private string m_sToolFilePath = string.Empty;
        private List<ImageInfo> m_aryImageInfo = new List<ImageInfo>();
        private List<ImageInfo> m_aryFilteredInfo = new List<ImageInfo>();
        private List<string> m_aryProducts = new List<string>();
        private WebClient m_webCtrl = new WebClient(); WebClient m_webCtrlForTool = new WebClient();
        private Timer m_timer = new Timer();

        public frmMain()
        {            
            InitializeComponent();

            InitWorkingFolder();
            InitSourceContents();
            InitToolFile();
            InitProductInfo();

            m_webCtrl.DownloadProgressChanged += new DownloadProgressChangedEventHandler(WClient_DownloadProgressChanged);
            m_webCtrl.DownloadFileCompleted += new AsyncCompletedEventHandler(WClient_DownloadFileCompleted);
            
            
            m_timer.Interval = 5000;
            m_timer.Tick += new EventHandler(m_timer_Tick);
        }

        private void m_timer_Tick(object sender, EventArgs e)
        {
            try
            {
                if (m_nPercentage > m_nPrevPercentage)
                {
                    int nRemindTime = -1;
                    long v = 0;
                    if (m_stopwatch.ElapsedMilliseconds > 0)
                    {
                        v = m_lnRecv / m_stopwatch.ElapsedMilliseconds;
                        nRemindTime = (int)((m_lnTotal - m_lnRecv) / v / 1000);
                    }
                    m_nPrevPercentage = m_nPercentage;
                    if (nRemindTime > 0)
                    {
                        WriteLog(string.Format("다운로드 중 ... {0}% ( {1:#,0}Kb / {2:#,0}Kb : {3}초 남음)"
                            , m_nPercentage
                            , m_lnRecv / 1024
                            , m_lnTotal / 1024, nRemindTime));
                    }
                    else
                    {
                        WriteLog(string.Format("다운로드 중 ... {0}% ( {1:#,0}Kb / {2:#,0}Kb"
                            , m_nPercentage
                            , m_lnRecv / 1024
                            , m_lnTotal / 1024));

                    }
                }
            }
            catch (Exception ex)
            {
                Logger.Current.Write(ex);
            }
        }

        private void InitToolFile()
        {
            try
            {
                if (File.Exists(m_sToolFilePath) == false)
                {
                    Uri uri = new Uri("http://www.hind.pe.kr/attachment/cfile1.uf@031B7843507F91551911D4.zip");
                    m_webCtrlForTool.DownloadFileAsync(uri, m_sToolFilePath);
                }
            }
            catch (Exception ex)
            {
                Logger.Current.Write(ex);
            }

        }

        private void InitWorkingFolder()
        {
            string sNewPath = Environment.GetFolderPath(Environment.SpecialFolder.MyDocuments);
            m_sPath = Path.Combine(sNewPath, "NexusUpdater");
            if (Directory.Exists(m_sPath) == false)
            {
                Directory.CreateDirectory(m_sPath);
            }
            Logger.Current.WorkingFolder = m_sPath;
            m_sToolFilePath = Path.Combine(m_sPath, "adb.zip");

        }

        private void InitSourceContents()
        {
            try
            {
                HttpWebRequest req = (HttpWebRequest)HttpWebRequest.Create(SRC_URL);
                StreamReader reader = new StreamReader(req.GetResponse().GetResponseStream());
                m_sPageSrc = reader.ReadToEnd();
                reader.Close();

                InitProductInfo();
                InitImageInfo();
            }
            catch (Exception ex)
            {
                Logger.Current.Write(ex);
            }
        }

        private void InitProductInfo()
        {
            try
            {
                m_aryProducts.Clear();
                Regex regexProducts = new Regex("Factory Images \"[a-zA-Z]+\" for (?<name>[A-Za-z0-9 ]+)");
                MatchCollection matchesProducts = regexProducts.Matches(m_sPageSrc);
                foreach (Match matchProduct in matchesProducts)
                {
                    string sProduct = matchProduct.Groups["name"].Value;
                    if (m_aryProducts.Contains(sProduct) == false)
                    {
                        m_aryProducts.Add(matchProduct.Groups["name"].Value);
                    }
                }

                cmbFilters.DataSource = m_aryProducts;
                string sDefaultProductName = m_aryProducts.Find(prod => prod.Contains("Nexus S"));
                if (string.IsNullOrEmpty(sDefaultProductName) == false)
                {
                    int nSelectedIndex = m_aryProducts.IndexOf(sDefaultProductName);
                    cmbFilters.SelectedIndex = nSelectedIndex;
                    ApplyFilter();
                }
            }
            catch (Exception ex)
            {
                Logger.Current.Write(ex);
            }

            
        }

        private void InitImageInfo()
        {
            m_aryImageInfo.Clear();
            m_aryFilteredInfo.Clear();

            try
            {
                Regex regexTitle = new Regex(@"<h2 id=""[a-z0-9]+"">(?<title>[0-9A-Za-z,/""()+ ]+)</h2>", RegexOptions.Singleline);
                Regex regexTable = new Regex(@"<table>", RegexOptions.Singleline);
                Regex regexTR = new Regex(@"<tr id=""(?<ver>[a-z0-9]+)"">", RegexOptions.Singleline);
               
                MatchCollection matchesTitle = regexTitle.Matches(m_sPageSrc);

                string sLine = string.Empty;

                foreach (Match matchTitle in matchesTitle)
                {
                    int nNextStringIndex = matchTitle.Index + matchTitle.Length;
                    StringReader reader = new StringReader(m_sPageSrc.Substring(nNextStringIndex));
                    sLine = reader.ReadLine();
                    sLine = reader.ReadLine();
                    sLine = reader.ReadLine();
                    if (regexTable.IsMatch(sLine))
                    {
                        while (Regex.IsMatch(sLine, @"\</table\>") == false)
                        {
                            sLine = reader.ReadLine();
                            Match matchID = regexTR.Match(sLine);
                            if (matchID.Success)
                            {
                                List<string> aryContents = new List<string>();
                                aryContents.Add(reader.ReadLine());
                                aryContents.Add(reader.ReadLine());
                                aryContents.Add(reader.ReadLine());
                                aryContents.Add(reader.ReadLine());
                                ImageInfo newImage = new ImageInfo();
                                if (newImage.Load(m_sPath,matchTitle.Groups["title"].Value,matchID.Groups["ver"].Value,aryContents))
                                    m_aryImageInfo.Add(newImage);
                            }
                        }
                    }
                }
            }
            catch(Exception ex)
            {
                Logger.Current.Write(ex);
            }

        }

        private void ApplyFilter()
        {
            try
            {
                string sProductName = (string)cmbFilters.SelectedItem;
                if (string.IsNullOrEmpty(sProductName) == false)
                {
                    m_aryFilteredInfo = m_aryImageInfo.FindAll(imgInfo => imgInfo.Name.Contains(sProductName));
                    m_aryFilteredInfo.Sort((imgPrev, imgNext) => imgNext.ID.CompareTo(imgPrev.ID));
                    cmbImages.DataSource = m_aryFilteredInfo;
                    if (cmbImages.Items.Count > 0)
                        cmbImages.SelectedIndex = 0;

                }
            }
            catch (Exception ex)
            {
                Logger.Current.Write(ex);
            }
        }

        private void WClient_DownloadFileCompleted(object sender, AsyncCompletedEventArgs e)
        {
            try
            {
                int nTotalElpTime = (int)(m_stopwatch.ElapsedMilliseconds / 1000);
                m_stopwatch.Stop();
                if (e.Error == null)
                {
                    WriteLog(string.Format("다운로드 완료... 총 {0}초 소요", nTotalElpTime));
                    ImageInfo img = (ImageInfo)cmbImages.SelectedItem;
                    if (img != null)
                    {
                        workerExtractFiles.RunWorkerAsync(img);
                    }
                }
                else
                {
                    WriteLog("다운로드는 완료했으나, 오류가 있습니다.");
                    WriteLog(e.Error.Message);
                }
                m_timer.Stop();
            }
            catch (Exception ex)
            {
                Logger.Current.Write(ex);
            }
        }

        private void WClient_DownloadProgressChanged(object sender, DownloadProgressChangedEventArgs e)
        {
            
            m_nP = e.ProgressPercentage;
            m_nPercentage = e.ProgressPercentage;
            m_lnRecv = e.BytesReceived;
            m_lnTotal = e.TotalBytesToReceive;

        }

        private void cmbFilters_SelectedIndexChanged(object sender, EventArgs e)
        {
            ApplyFilter();
        }       

        private void cmbImages_SelectedIndexChanged(object sender, EventArgs e)
        {
            ImageInfo img = (ImageInfo)cmbImages.SelectedItem;
            m_selectedImage = img;
            if (m_selectedImage != null)
            {
                btnDownload.Enabled = m_selectedImage.IsDownloaded == false;
                btnUpdate.Enabled = m_selectedImage.IsDownloaded;
            }
        }

        private void btnDownload_Click(object sender, EventArgs e)
        {
            ImageInfo img = m_selectedImage;
            if (img != null)
            {
                try
                {
                    m_nP = -1;
                    m_timer.Start();
                    btnDownload.Enabled = false;
                    btnUpdate.Enabled = false;
                    cmbImages.Enabled = false;
                    cmbFilters.Enabled = false;
                    Uri uri = new Uri(img.Url);
                    m_sFilename = Path.Combine(m_sPath, img.Filename);
                    m_stopwatch.Reset();
                    m_stopwatch.Start();
                    m_webCtrl.DownloadFileAsync(uri, m_sFilename);
                }
                catch (Exception ex)
                {
                    Logger.Current.Write(ex);
                }
            }
        }

        private void btnUpdate_Click(object sender, EventArgs e)
        {
            try
            {
                ImageInfo img = m_selectedImage;


                string sExtractedPath = Path.Combine(m_sPath, img.ExtractedPath);
                string sFlashAllBat = Path.Combine(sExtractedPath, "flash-all.bat");
                string sFastboot = Path.Combine(sExtractedPath, "fastboot.exe");

                if (img != null && File.Exists(img.FullFilename))
                {
                    if (img.IsHasExtracted == false)
                    {
                        workerExtractFiles.RunWorkerAsync(img);
                    }

                }

                ConfirmDlg dlg = new ConfirmDlg();
                dlg.BasePath = sExtractedPath;

                dlg.Show();
            }
            catch (Exception ex)
            {
                Logger.Current.Write(ex);
            }
        }

        private void workerExtractFiles_DoWork(object sender, DoWorkEventArgs e)
        {
            try
            {
                ImageInfo img = (ImageInfo)e.Argument;
                FileInfo fileInfo = new FileInfo(img.FullFilename);


                string sNewTarFilename = fileInfo.Name.Replace(fileInfo.Extension, ".tar");
                sNewTarFilename = Path.Combine(m_sPath, sNewTarFilename);
                workerExtractFiles.ReportProgress(0);
                using (Stream inStream = new GZipInputStream(File.Open(img.FullFilename, FileMode.Open)))
                using (FileStream outStream = File.Create(sNewTarFilename))
                {
                    byte[] buffer = new byte[4096];
                    StreamUtils.Copy(inStream, outStream, buffer);
                }
                workerExtractFiles.ReportProgress(50);

                using (TarArchive tarAchive = TarArchive.CreateInputTarArchive(File.Open(sNewTarFilename, FileMode.Open)))
                {
                    tarAchive.ExtractContents(m_sPath);
                }
                workerExtractFiles.ReportProgress(100);

                string[] aryFiles = Directory.GetFiles(m_sPath, "*.tar");
                foreach (string sPath in aryFiles)
                {
                    File.Delete(sPath);
                }

                string sExtractedPath = Path.Combine(m_sPath, img.ExtractedPath);

                ZipFile zf = null;
                try
                {
                    FileStream fs = File.OpenRead(m_sToolFilePath);
                    zf = new ZipFile(fs);
                    foreach (ZipEntry zipEntry in zf)
                    {
                        if (!zipEntry.IsFile)
                        {
                            continue;           // Ignore directories
                        }
                        String entryFileName = zipEntry.Name;
                        // to remove the folder from the entry:- entryFileName = Path.GetFileName(entryFileName);
                        // Optionally match entrynames against a selection list here to skip as desired.
                        // The unpacked length is available in the zipEntry.Size property.

                        byte[] buffer = new byte[4096];     // 4K is optimum
                        Stream zipStream = zf.GetInputStream(zipEntry);

                        // Manipulate the output filename here as desired.
                        String fullZipToPath = Path.Combine(sExtractedPath, entryFileName);
                        string directoryName = Path.GetDirectoryName(fullZipToPath);
                        if (directoryName.Length > 0)
                            Directory.CreateDirectory(directoryName);

                        // Unzip file in buffered chunks. This is just as fast as unpacking to a buffer the full size
                        // of the file, but does not waste memory.
                        // The "using" will close the stream even if an exception occurs.
                        using (FileStream streamWriter = File.Create(fullZipToPath))
                        {
                            StreamUtils.Copy(zipStream, streamWriter, buffer);
                        }
                    }
                }
                finally
                {
                    if (zf != null)
                    {
                        zf.IsStreamOwner = true; // Makes close also shut the underlying stream
                        zf.Close(); // Ensure we release resources
                    }
                }
                e.Result = img;
            }
            catch (Exception ex)
            {
                Logger.Current.Write(ex);
            }
        }

        private void workerExtractFiles_ProgressChanged(object sender, ProgressChangedEventArgs e)
        {
            switch (e.ProgressPercentage)
            {
                case 0:
                    WriteLog("다운받은 롬 파일의 압축을 풀고 있습니다.(TGZ -> TAR)");
                    break;
                case 50:
                    WriteLog("압축풀린 TAR 파일의 내용을 풀고 있습니다.(TAR )");
                    break;
                case 100:
                    WriteLog("모든 내용을 작업용 폴더에 풀어놓았습니다.");
                    break;
            }
        }

        private void workerExtractFiles_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            try
            {
                ImageInfo img = (ImageInfo)e.Result;
                string sPath = Path.Combine(m_sPath, img.ExtractedPath);

                if (Directory.Exists(sPath) && img.IsDownloaded)
                {
                    int nSelectedIdx = cmbImages.SelectedIndex;
                    cmbImages.DataSource = null;
                    cmbImages.DataSource = m_aryFilteredInfo;
                    cmbImages.SelectedIndex = nSelectedIdx;

                    cmbImages.Enabled = true;
                    cmbFilters.Enabled = true;

                    WriteLog("업데이트 준비가 완료되었습니다.");
                }

                string sExtractedPath = Path.Combine(m_sPath, img.ExtractedPath);
                string sFlashAllBat = Path.Combine(sExtractedPath, "flash-all.bat");
                string sFastboot = Path.Combine(sExtractedPath, "fastboot.exe");

                ConfirmDlg dlg = new ConfirmDlg();
                dlg.BasePath = sExtractedPath;

                dlg.Show();
            }
            catch (Exception ex)
            {
                Logger.Current.Write(ex);
            }
        }

      
        void WriteLog(string sMessage)
        {
            txtLog.AppendText(Environment.NewLine);
            txtLog.AppendText(DateTime.Now.ToString("HH:mm:ss"));
            txtLog.AppendText("  ");
            txtLog.AppendText(sMessage);
            txtLog.Select(txtLog.TextLength - 1, txtLog.TextLength - 1);
            txtLog.ScrollToCaret();

            Logger.Current.Write(sMessage);
        }


    }
}

